package Spring;

import java.awt.geom.Line2D;
import java.awt.geom.Point2D;
import java.util.Iterator;
import java.util.NoSuchElementException;
import java.util.Vector;

public class Spring{

	private Vector<Point> points;
	private Vector<Point> vertices;
	
	public Spring() {
		points=new Vector<Point>();
		vertices=new Vector<Point>();
	}
	
	public void addPoint(double x){
		for(int i=1;i<=x;i++){
			points.add(new Point(15-30*Math.random(),15-30*Math.random()));
		}
	}
	public void addVertice(int p1, int p2){
		if((p1<=points.size() && p1>0)&&(p2<=points.size() && p2>0)&&(p1!=p2)){
			vertices.add(new Point((double)p1,(double)p2));
		}
	}
	public void shake(){
		for(Point point:points){
			point.x+=10-20*Math.random();
			point.y+=10-20*Math.random();
		}
	}
	
	public Iterator<Line2D> iterVertices() {
		return new IterVertices();
	}
	public Iterator<Point2D> iterPoints() {
		return new IterPoints();
	}
	
	public void doStep(){
		//constantes de reglage ...
		//c1 increases the repulsive force
		//c2 increases the attractive force
		//delta multiplies the total force (little delta->little steps)
		double c2=.2,c1=10,delta=.4;
		int i=1;
		for (Point p1 : points) {
			int j=1;
			double fx=0,fy=0;
			for (Point p2 : points) {
				if(i!=j){
					//force de répulsion
					//repulsive force
					Point rep=new Point();
					rep.x=c1*(p1.x-p2.x)/Math.pow(p1.dist(p2),2);
					rep.y=c1*(p1.y-p2.y)/Math.pow(p1.dist(p2),2);
					
					//force d'attraction valide si on a un arc entre 2 sommets
					//attraction is only applied if an edge exists between the nodes p1 and p2
					Point att=new Point();
					for(Point ver:vertices){	
						if((ver.x==i && ver.y==j)||(ver.y==i && ver.x==j)){
							att.x=c2*Math.log(p1.dist(p2)/10)*(p2.x-p1.x);
							att.y=c2*Math.log(p1.dist(p2)/10)*(p2.y-p1.y);
						}
					}
					//on concatène les forces ...
					//sum of repulsive and attractive forces
					fx+=rep.x+att.x;
					fy+=rep.y+att.y;
					j++;
				}
			}
			//deplacement du point p1 en fonction de la force calculée
			//We move the node according to force f
			p1.x+=.5*fx;
			p1.y+=.5*fy;
			i++;
		}
	}
	
	private class Point {
		public double x,y;
		public Point(){x=y=0;}
		public Point(double x,double y){this.x=x;this.y=y;}
		public double dist(Point p){ //distance entre moi et p
			return Math.sqrt(Math.pow((p.x-x),2)+Math.pow((p.y-y),2));
		}
	}
	
	private class IterVertices implements Iterator<Line2D>{	
		private int pos=0;
		public IterVertices() {}
		public boolean hasNext() {return(pos<vertices.size());}
		public Line2D next() {
			if (pos>=vertices.size())
		         throw new NoSuchElementException();
			Point p=vertices.elementAt(pos);
			Line2D res=new Line2D.Double(points.elementAt((int)p.x-1).x,points.elementAt((int)p.x-1).y,points.elementAt((int)p.y-1).x,points.elementAt((int)p.y-1).y);
			pos++;
			return res;
		}	
		public void remove() {throw new UnsupportedOperationException();}	
	}
	private class IterPoints implements Iterator<Point2D>{	
		private int pos=0;
		public IterPoints() {}
		public boolean hasNext() {return(pos<points.size() );}
		public Point2D next() {
			if (pos>=points.size())
		         throw new NoSuchElementException();
			Point2D res=new Point2D.Double(points.elementAt(pos).x,points.elementAt(pos).y);
			pos++;
			return res;
		}	
		public void remove() {throw new UnsupportedOperationException();}	
	}
	
	
}
